open Prefs
open Debug
open Tptp
open Terms
open Print
open Unix
open Status

let name = "AProVE";;

let default_cmd = "aprove";;

let read_answer (ic,oc) = 
  let ans = input_line ic in
    ignore (input_line ic); 
    ignore (input_line ic); 
    ans
;;

let rec tell (ic,oc) = function
    msg::msgs -> 
      debug_if TRACE (!debug_checker) msg;
      log (lazy msg);
      output_string oc msg;
      output_char oc '\n';
      tell (ic,oc) msgs; 
  | [] -> 
      output_string oc ".\n";
      flush oc;
;;

let ask (ic,oc) = 
  match read_answer (ic,oc) with 
      "true"|"YES" -> TERM_YES
    | "false"|"NO" -> TERM_NO
    | "maybe"|"MAYBE" -> TERM_UNKNOWN
    | "timed out"|"TIMEOUT" -> TERM_UNKNOWN
    | x -> failwith ("Error communicating with " ^ name ^ 
                       ": responded with " ^ x)
;;

let start cmd = 
  let ic,oc = Unix.open_process cmd in
    set_binary_mode_in ic false ; 
    set_binary_mode_out oc false ;
    (ic,oc)
;;

let stop (ic,oc) = 
  close_in ic;
  ignore (Unix.close_process(ic,oc))
;;

let left_handle = ref None;;
let right_handle = ref None;;

let set_left_handle cmd h =
  left_handle := Some(cmd,h)
;;

let set_right_handle cmd h =
  right_handle := Some(cmd,h)
;;

let set_handle cmd h = set_left_handle cmd h; h
;;

let get_handle cmd = 
  match !left_handle with 
      None -> 
        debug_if_l TRACE (!debug_checker) 
          (lazy ("Starting AProVE (" ^ cmd ^ ") ..."));
        let h = start cmd in
          set_handle cmd h
    | Some(s,h) when s = cmd -> h
    | Some(s,h) -> 
        debug_if_l TRACE (!debug_checker) 
          (lazy ("Restarting AProVE (" ^ cmd ^ ") ..."));
        stop h;
        let h' = start cmd in
          set_handle cmd h'
;;

let get_both_handles cmd =
  let h1 = 
    match !left_handle with 
        None -> 
          debug_if_l TRACE (!debug_checker) 
            (lazy ("Starting AProVE-1 (" ^ cmd ^ ") ..."));
          let h = start cmd in
            set_left_handle cmd h ; h
      | Some(s,h) when s = cmd -> h
      | Some(s,h) -> 
          debug_if_l TRACE (!debug_checker) 
            (lazy ("Restarting AProVE-1 (" ^ cmd ^ ") ..."));
          stop h;
          let h' = start cmd in
            set_left_handle cmd h' ; h'
  in
  let h2 = 
    match !right_handle with 
        None -> 
          debug_if_l TRACE (!debug_checker) 
            (lazy ("Starting AProVE-2 (" ^ cmd ^ ") ..."));
          let h = start cmd in
            set_right_handle cmd h ; h
      | Some(s,h) when s = cmd -> h
      | Some(s,h) -> 
          debug_if_l TRACE (!debug_checker) 
            (lazy ("Restarting AProVE-2 (" ^ cmd ^ ") ..."));
          stop h;
          let h' = start cmd in
            set_right_handle cmd h' ; h'
  in
    [h1;h2]

(*       None ->  *)
(*         debug_if_l TRACE (!debug_checker)  *)
(*           (lazy ("Starting AProVE (" ^ cmd ^ ") ...")); *)
(*         let h = start cmd in *)
(*           set_handle cmd h *)
(*     | Some(s,h) when s = cmd -> h *)
(*     | Some(s,h) ->  *)
(*         debug_if_l TRACE (!debug_checker)  *)
(*           (lazy ("Restarting AProVE (" ^ cmd ^ ") ...")); *)
(*         stop h; *)
(*         let h' = start cmd in *)
(*           set_handle cmd h' *)
;;

let get_handles n cmd = 
  if n > 1 then get_both_handles cmd else
    [get_handle cmd]
;;


let shutdown () = 
  begin 
    match !left_handle with 
        Some(cmd,h) -> 
          stop h
      | None -> ()
  end;
  match !right_handle with 
      Some(cmd,h) -> 
        stop h
    | None -> ()
;;
